@@ -74,7 +74,6 @@
 #include "stylehelper.h"
 #include "stylesheetloader.h"
 #include "systemtrayicon.h"
-#include "windows7thumbbar.h"
 #include "application.h"
 #include "database.h"
 #include "player.h"
@@ -122,7 +121,7 @@
 #include "covermanager/albumcovermanager.h"
 #include "covermanager/albumcoverchoicecontroller.h"
 #include "covermanager/albumcoverloader.h"
-#include "covermanager/currentartloader.h"
+#include "covermanager/currentalbumcoverloader.h"
 #ifndef Q_OS_WIN
 #  include "device/devicemanager.h"
 #  include "device/devicestatefiltermodel.h"
@@ -135,12 +134,22 @@
 #include "settings/backendsettingspage.h"
 #include "settings/playlistsettingspage.h"
 #ifdef HAVE_TIDAL
+#  include "tidal/tidalservice.h"
 #  include "settings/tidalsettingspage.h"
 #endif
+#ifdef HAVE_QOBUZ
+#  include "qobuz/qobuzservice.h"
+#  include "settings/qobuzsettingspage.h"
+#endif
+#ifdef HAVE_SUBSONIC
+#  include "subsonic/subsonicservice.h"
+#  include "settings/subsonicsettingspage.h"
+#endif
 
 #include "internet/internetservices.h"
 #include "internet/internetservice.h"
-#include "internet/internetsearchview.h"
+#include "internet/internetsongsview.h"
+#include "internet/internettabsview.h"
 
 #include "scrobbler/audioscrobbler.h"
 
@@ -157,6 +166,10 @@
 #  include "moodbar/moodbarproxystyle.h"
 #endif
 
+#ifdef Q_OS_WIN
+#  include "windows7thumbbar.h"
+#endif
+
 using std::bind;
 using std::floor;
 using std::stable_sort;
@@ -172,7 +185,9 @@ const int kTrackPositionUpdateTimeMs = 1000;
 MainWindow::MainWindow(Application *app, SystemTrayIcon *tray_icon, OSD *osd, const CommandlineOptions &options, QWidget *parent) :
       QMainWindow(parent),
       ui_(new Ui_MainWindow),
+#ifdef Q_OS_WIN
       thumbbar_(new Windows7ThumbBar(this)),
+#endif
       app_(app),
       tray_icon_(tray_icon),
       osd_(osd),
@@ -208,7 +223,13 @@ MainWindow::MainWindow(Application *app, SystemTrayIcon *tray_icon, OSD *osd, co
         return dialog;
       }),
 #ifdef HAVE_TIDAL
-      tidal_search_view_(new InternetSearchView(app_, app_->tidal_search(), TidalSettingsPage::kSettingsGroup, SettingsDialog::Page_Tidal, this)),
+      tidal_view_(new InternetTabsView(app_, app->internet_services()->ServiceBySource(Song::Source_Tidal), app_->tidal_search(), TidalSettingsPage::kSettingsGroup, SettingsDialog::Page_Tidal, this)),
+#endif
+#ifdef HAVE_QOBUZ
+      qobuz_view_(new InternetTabsView(app_, app->internet_services()->ServiceBySource(Song::Source_Qobuz), app_->qobuz_search(), QobuzSettingsPage::kSettingsGroup, SettingsDialog::Page_Qobuz, this)),
+#endif
+#ifdef HAVE_SUBSONIC
+      subsonic_view_(new InternetSongsView(app_, app->internet_services()->ServiceBySource(Song::Source_Subsonic), SubsonicSettingsPage::kSettingsGroup, SettingsDialog::Page_Subsonic, this)),
 #endif
       playlist_menu_(new QMenu(this)),
       playlist_add_to_another_(nullptr),
@@ -221,7 +242,8 @@ MainWindow::MainWindow(Application *app, SystemTrayIcon *tray_icon, OSD *osd, co
       playing_widget_(true),
       doubleclick_addmode_(BehaviourSettingsPage::AddBehaviour_Append),
       doubleclick_playmode_(BehaviourSettingsPage::PlayBehaviour_Never),
-      menu_playmode_(BehaviourSettingsPage::PlayBehaviour_Never)
+      menu_playmode_(BehaviourSettingsPage::PlayBehaviour_Never),
+      exit_count_(0)
       {
 
   qLog(Debug) << "Starting";
@@ -231,12 +253,9 @@ MainWindow::MainWindow(Application *app, SystemTrayIcon *tray_icon, OSD *osd, co
 
   // Initialise the UI
   ui_->setupUi(this);
-#ifdef Q_OS_MACOS
-  ui_->menu_help->menuAction()->setVisible(false);
-#endif
 
-  connect(app_->current_art_loader(), SIGNAL(ArtLoaded(Song, QString, QImage)), SLOT(AlbumArtLoaded(Song, QString, QImage)));
-  album_cover_choice_controller_->SetApplication(app);
+  connect(app_->current_albumcover_loader(), SIGNAL(AlbumCoverLoaded(Song, QUrl, QImage)), SLOT(AlbumCoverLoaded(Song, QUrl, QImage)));
+  album_cover_choice_controller_->Init(app);
   connect(album_cover_choice_controller_->cover_from_file_action(), SIGNAL(triggered()), this, SLOT(LoadCoverFromFile()));
   connect(album_cover_choice_controller_->cover_to_file_action(), SIGNAL(triggered()), this, SLOT(SaveCoverToFile()));
   connect(album_cover_choice_controller_->cover_from_url_action(), SIGNAL(triggered()), this, SLOT(LoadCoverFromURL()));
@@ -246,8 +265,8 @@ MainWindow::MainWindow(Application *app, SystemTrayIcon *tray_icon, OSD *osd, co
   connect(album_cover_choice_controller_->search_cover_auto_action(), SIGNAL(triggered()), this, SLOT(SearchCoverAutomatically()));
 
   ui_->multi_loading_indicator->SetTaskManager(app_->task_manager());
-  context_view_->SetApplication(app_, collection_view_->view(), album_cover_choice_controller_);
-  ui_->widget_playing->SetApplication(app_, album_cover_choice_controller_);
+  context_view_->Init(app_, collection_view_->view(), album_cover_choice_controller_);
+  ui_->widget_playing->Init(app_, album_cover_choice_controller_);
 
   // Initialise the search widget
   StyleHelper::setBaseColor(palette().color(QPalette::Highlight).darker());
@@ -262,7 +281,13 @@ MainWindow::MainWindow(Application *app, SystemTrayIcon *tray_icon, OSD *osd, co
   ui_->tabs->AddTab(device_view_, "devices", IconLoader::Load("device"), tr("Devices"));
 #endif
 #ifdef HAVE_TIDAL
-  ui_->tabs->AddTab(tidal_search_view_, "tidal", IconLoader::Load("tidal"), tr("Tidal"));
+  ui_->tabs->AddTab(tidal_view_, "tidal", IconLoader::Load("tidal"), tr("Tidal"));
+#endif
+#ifdef HAVE_QOBUZ
+  ui_->tabs->AddTab(qobuz_view_, "qobuz", IconLoader::Load("qobuz"), tr("Qobuz"));
+#endif
+#ifdef HAVE_SUBSONIC
+  ui_->tabs->AddTab(subsonic_view_, "subsonic", IconLoader::Load("subsonic"), tr("Subsonic"));
 #endif
 
   // Add the playing widget to the fancy tab widget
@@ -322,11 +347,11 @@ MainWindow::MainWindow(Application *app, SystemTrayIcon *tray_icon, OSD *osd, co
 
   ui_->action_open_file->setIcon(IconLoader::Load("document-open"));
   ui_->action_open_cd->setIcon(IconLoader::Load("cd"));
-  ui_->action_previous_track->setIcon(IconLoader::Load("media-rewind"));
+  ui_->action_previous_track->setIcon(IconLoader::Load("media-skip-backward"));
   ui_->action_play_pause->setIcon(IconLoader::Load("media-play"));
   ui_->action_stop->setIcon(IconLoader::Load("media-stop"));
   ui_->action_stop_after_this_track->setIcon(IconLoader::Load("media-stop"));
-  ui_->action_next_track->setIcon(IconLoader::Load("media-forward"));
+  ui_->action_next_track->setIcon(IconLoader::Load("media-skip-forward"));
   ui_->action_quit->setIcon(IconLoader::Load("application-exit"));
 
   // Playlist
@@ -358,7 +383,8 @@ MainWindow::MainWindow(Application *app, SystemTrayIcon *tray_icon, OSD *osd, co
 
   // Scrobble
 
-  ui_->action_toggle_scrobbling->setIcon(IconLoader::Load("scrobble-disabled", 22));
+  ui_->action_toggle_scrobbling->setIcon(IconLoader::Load("scrobble-disabled"));
+  ui_->action_love->setIcon(IconLoader::Load("love"));
 
   // File view connections
   connect(file_view_, SIGNAL(AddToPlaylist(QMimeData*)), SLOT(AddToPlaylist(QMimeData*)));
@@ -387,6 +413,7 @@ MainWindow::MainWindow(Application *app, SystemTrayIcon *tray_icon, OSD *osd, co
   connect(ui_->action_remove_unavailable, SIGNAL(triggered()), app_->playlist_manager(), SLOT(RemoveUnavailableCurrent()));
   connect(ui_->action_remove_from_playlist, SIGNAL(triggered()), SLOT(PlaylistRemoveCurrent()));
   connect(ui_->action_edit_track, SIGNAL(triggered()), SLOT(EditTracks()));
+  connect(ui_->action_rescan_songs, SIGNAL(triggered()), SLOT(RescanSongs()));
   connect(ui_->action_renumber_tracks, SIGNAL(triggered()), SLOT(RenumberTracks()));
   connect(ui_->action_selection_set_value, SIGNAL(triggered()), SLOT(SelectionSetValue()));
   connect(ui_->action_edit_value, SIGNAL(triggered()), SLOT(EditValue()));
@@ -411,6 +438,7 @@ MainWindow::MainWindow(Application *app, SystemTrayIcon *tray_icon, OSD *osd, co
   connect(ui_->action_jump, SIGNAL(triggered()), ui_->playlist->view(), SLOT(JumpToCurrentlyPlayingTrack()));
   connect(ui_->action_update_collection, SIGNAL(triggered()), app_->collection(), SLOT(IncrementalScan()));
   connect(ui_->action_full_collection_scan, SIGNAL(triggered()), app_->collection(), SLOT(FullScan()));
+  connect(ui_->action_abort_collection_scan, SIGNAL(triggered()), app_->collection(), SLOT(AbortScan()));
 #if defined(HAVE_GSTREAMER)
   connect(ui_->action_add_files_to_transcoder, SIGNAL(triggered()), SLOT(AddFilesToTranscoder()));
 #else
@@ -418,6 +446,7 @@ MainWindow::MainWindow(Application *app, SystemTrayIcon *tray_icon, OSD *osd, co
 #endif
 
   connect(ui_->action_toggle_scrobbling, SIGNAL(triggered()), app_->scrobbler(), SLOT(ToggleScrobbling()));
+  connect(ui_->action_love, SIGNAL(triggered()), SLOT(Love()));
   connect(app_->scrobbler(), SIGNAL(ErrorMessage(QString)), SLOT(ShowErrorDialog(QString)));
 
   // Playlist view actions
@@ -434,6 +463,7 @@ MainWindow::MainWindow(Application *app, SystemTrayIcon *tray_icon, OSD *osd, co
   ui_->pause_play_button->setDefaultAction(ui_->action_play_pause);
   ui_->stop_button->setDefaultAction(ui_->action_stop);
   ui_->button_scrobble->setDefaultAction(ui_->action_toggle_scrobbling);
+  ui_->button_love->setDefaultAction(ui_->action_love);
 
   ui_->playlist->SetActions(ui_->action_new_playlist, ui_->action_load_playlist, ui_->action_save_playlist, ui_->action_clear_playlist, ui_->action_next_playlist,    /* These two actions aren't associated */ ui_->action_previous_playlist /* to a button but to the main window */ );
 
@@ -467,6 +497,7 @@ MainWindow::MainWindow(Application *app, SystemTrayIcon *tray_icon, OSD *osd, co
   connect(app_->player(), SIGNAL(Stopped()), ui_->playlist, SLOT(ActiveStopped()));
 
   connect(app_->player(), SIGNAL(Paused()), osd_, SLOT(Paused()));
+  connect(app_->player(), SIGNAL(Resumed()), osd_, SLOT(Resumed()));
   connect(app_->player(), SIGNAL(Stopped()), osd_, SLOT(Stopped()));
   connect(app_->player(), SIGNAL(PlaylistFinished()), osd_, SLOT(PlaylistFinished()));
   connect(app_->player(), SIGNAL(VolumeChanged(int)), osd_, SLOT(VolumeChanged(int)));
@@ -493,10 +524,6 @@ MainWindow::MainWindow(Application *app, SystemTrayIcon *tray_icon, OSD *osd, co
   connect(ui_->track_slider, SIGNAL(Previous()), app_->player(), SLOT(Previous()));
   connect(ui_->track_slider, SIGNAL(Next()), app_->player(), SLOT(Next()));
 
-  // Context connections
-
-  connect(context_view_->albums(), SIGNAL(AddToPlaylistSignal(QMimeData*)), SLOT(AddToPlaylist(QMimeData*)));
-
   // Collection connections
   connect(collection_view_->view(), SIGNAL(AddToPlaylistSignal(QMimeData*)), SLOT(AddToPlaylist(QMimeData*)));
   connect(collection_view_->view(), SIGNAL(ShowConfigDialog()), SLOT(ShowCollectionConfig()));
@@ -544,23 +571,37 @@ MainWindow::MainWindow(Application *app, SystemTrayIcon *tray_icon, OSD *osd, co
   collection_view_->filter()->AddMenuAction(collection_config_action);
 
 #ifdef HAVE_TIDAL
-  connect(tidal_search_view_, SIGNAL(AddToPlaylist(QMimeData*)), SLOT(AddToPlaylist(QMimeData*)));
+  connect(tidal_view_->artists_collection_view(), SIGNAL(AddToPlaylistSignal(QMimeData*)), SLOT(AddToPlaylist(QMimeData*)));
+  connect(tidal_view_->albums_collection_view(), SIGNAL(AddToPlaylistSignal(QMimeData*)), SLOT(AddToPlaylist(QMimeData*)));
+  connect(tidal_view_->songs_collection_view(), SIGNAL(AddToPlaylistSignal(QMimeData*)), SLOT(AddToPlaylist(QMimeData*)));
+  connect(tidal_view_->search_view(), SIGNAL(AddToPlaylist(QMimeData*)), SLOT(AddToPlaylist(QMimeData*)));
+  if (TidalService *tidalservice = qobject_cast<TidalService*> (app_->internet_services()->ServiceBySource(Song::Source_Tidal)))
+    connect(this, SIGNAL(AuthorisationUrlReceived(const QUrl&)), tidalservice, SLOT(AuthorisationUrlReceived(const QUrl&)));
+#endif
+
+#ifdef HAVE_QOBUZ
+  connect(qobuz_view_->artists_collection_view(), SIGNAL(AddToPlaylistSignal(QMimeData*)), SLOT(AddToPlaylist(QMimeData*)));
+  connect(qobuz_view_->albums_collection_view(), SIGNAL(AddToPlaylistSignal(QMimeData*)), SLOT(AddToPlaylist(QMimeData*)));
+  connect(qobuz_view_->songs_collection_view(), SIGNAL(AddToPlaylistSignal(QMimeData*)), SLOT(AddToPlaylist(QMimeData*)));
+  connect(qobuz_view_->search_view(), SIGNAL(AddToPlaylist(QMimeData*)), SLOT(AddToPlaylist(QMimeData*)));
+#endif
+
+#ifdef HAVE_SUBSONIC
+  connect(subsonic_view_->view(), SIGNAL(AddToPlaylistSignal(QMimeData*)), SLOT(AddToPlaylist(QMimeData*)));
 #endif
 
   // Playlist menu
+  connect(playlist_menu_, SIGNAL(aboutToHide()), SLOT(PlaylistMenuHidden()));
   playlist_play_pause_ = playlist_menu_->addAction(tr("Play"), this, SLOT(PlaylistPlay()));
   playlist_menu_->addAction(ui_->action_stop);
   playlist_stop_after_ = playlist_menu_->addAction(IconLoader::Load("media-stop"), tr("Stop after this track"), this, SLOT(PlaylistStopAfter()));
   playlist_queue_ = playlist_menu_->addAction(IconLoader::Load("go-next"), tr("Toggle queue status"), this, SLOT(PlaylistQueue()));
-  playlist_queue_->setVisible(false);
   playlist_queue_->setShortcut(QKeySequence("Ctrl+D"));
   ui_->playlist->addAction(playlist_queue_);
   playlist_queue_play_next_ = playlist_menu_->addAction(IconLoader::Load("go-next"), tr("Queue selected tracks to play next"), this, SLOT(PlaylistQueuePlayNext()));
   playlist_queue_play_next_->setShortcut(QKeySequence("Ctrl+Shift+D"));
-  playlist_queue_play_next_->setVisible(false);
   ui_->playlist->addAction(playlist_queue_play_next_);
-  playlist_skip_ = playlist_menu_->addAction(IconLoader::Load("media-forward"), tr("Toggle skip status"), this, SLOT(PlaylistSkip()));
-  playlist_skip_->setVisible(false);
+  playlist_skip_ = playlist_menu_->addAction(IconLoader::Load("media-skip-forward"), tr("Toggle skip status"), this, SLOT(PlaylistSkip()));
   ui_->playlist->addAction(playlist_skip_);
 
   playlist_menu_->addSeparator();
@@ -571,6 +612,7 @@ MainWindow::MainWindow(Application *app, SystemTrayIcon *tray_icon, OSD *osd, co
   playlist_menu_->addAction(ui_->action_renumber_tracks);
   playlist_menu_->addAction(ui_->action_selection_set_value);
   playlist_menu_->addAction(ui_->action_auto_complete_tags);
+  playlist_menu_->addAction(ui_->action_rescan_songs);
 #ifdef HAVE_GSTREAMER
   playlist_menu_->addAction(ui_->action_add_files_to_transcoder);
 #endif
@@ -607,13 +649,14 @@ MainWindow::MainWindow(Application *app, SystemTrayIcon *tray_icon, OSD *osd, co
 
   connect(app_->scrobbler(), SIGNAL(ScrobblingEnabledChanged(bool)), SLOT(ScrobblingEnabledChanged(bool)));
   connect(app_->scrobbler(), SIGNAL(ScrobbleButtonVisibilityChanged(bool)), SLOT(ScrobbleButtonVisibilityChanged(bool)));
+  connect(app_->scrobbler(), SIGNAL(LoveButtonVisibilityChanged(bool)), SLOT(LoveButtonVisibilityChanged(bool)));
 
 #ifdef Q_OS_MACOS
   mac::SetApplicationHandler(this);
 #endif
   // Tray icon
   if (tray_icon_) {
-    tray_icon_->SetupMenu(ui_->action_previous_track, ui_->action_play_pause, ui_->action_stop, ui_->action_stop_after_this_track, ui_->action_next_track, ui_->action_mute, ui_->action_quit);
+    tray_icon_->SetupMenu(ui_->action_previous_track, ui_->action_play_pause, ui_->action_stop, ui_->action_stop_after_this_track, ui_->action_next_track, ui_->action_mute, ui_->action_love, ui_->action_quit);
     connect(tray_icon_, SIGNAL(PlayPause()), app_->player(), SLOT(PlayPause()));
     connect(tray_icon_, SIGNAL(SeekForward()), app_->player(), SLOT(SeekForward()));
     connect(tray_icon_, SIGNAL(SeekBackward()), app_->player(), SLOT(SeekBackward()));
@@ -624,7 +667,9 @@ MainWindow::MainWindow(Application *app, SystemTrayIcon *tray_icon, OSD *osd, co
   }
 
   // Windows 7 thumbbar buttons
-  thumbbar_->SetActions(QList<QAction*>() << ui_->action_previous_track << ui_->action_play_pause << ui_->action_stop << ui_->action_next_track << nullptr); // spacer
+#ifdef Q_OS_WIN
+  thumbbar_->SetActions(QList<QAction*>() << ui_->action_previous_track << ui_->action_play_pause << ui_->action_stop << ui_->action_next_track << nullptr << ui_->action_love);
+#endif
 
 #if (defined(Q_OS_MACOS) && defined(HAVE_SPARKLE))
   // Add check for updates item to application menu.
@@ -651,6 +696,7 @@ MainWindow::MainWindow(Application *app, SystemTrayIcon *tray_icon, OSD *osd, co
   connect(global_shortcuts_, SIGNAL(ShowOSD()), app_->player(), SLOT(ShowOSD()));
   connect(global_shortcuts_, SIGNAL(TogglePrettyOSD()), app_->player(), SLOT(TogglePrettyOSD()));
   connect(global_shortcuts_, SIGNAL(ToggleScrobbling()), app_->scrobbler(), SLOT(ToggleScrobbling()));
+  connect(global_shortcuts_, SIGNAL(Love()), app_->scrobbler(), SLOT(Love()));
 #endif
 
   // Fancy tabs
@@ -658,16 +704,24 @@ MainWindow::MainWindow(Application *app, SystemTrayIcon *tray_icon, OSD *osd, co
 
   // Context
   connect(app_->playlist_manager(), SIGNAL(CurrentSongChanged(Song)), context_view_, SLOT(SongChanged(Song)));
+  connect(app_->playlist_manager(), SIGNAL(SongMetadataChanged(Song)), context_view_, SLOT(SongChanged(Song)));
   connect(app_->player(), SIGNAL(PlaylistFinished()), context_view_, SLOT(Stopped()));
   connect(app_->player(), SIGNAL(Playing()), context_view_, SLOT(Playing()));
   connect(app_->player(), SIGNAL(Stopped()), context_view_, SLOT(Stopped()));
   connect(app_->player(), SIGNAL(Error()), context_view_, SLOT(Error()));
+  connect(this, SIGNAL(AlbumCoverReady(Song, QUrl, QImage)), context_view_, SLOT(AlbumCoverLoaded(Song, QUrl, QImage)));
+  connect(this, SIGNAL(SearchCoverInProgress()), context_view_, SLOT(SearchCoverInProgress()));
+  connect(context_view_->albums(), SIGNAL(AddToPlaylistSignal(QMimeData*)), SLOT(AddToPlaylist(QMimeData*)));
 
   // Analyzer
   connect(ui_->analyzer, SIGNAL(WheelEvent(int)), SLOT(VolumeWheelEvent(int)));
 
   // Statusbar widgets
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 11, 0))
+  ui_->playlist_summary->setMinimumWidth(QFontMetrics(font()).horizontalAdvance("WW selected of WW tracks - [ WW:WW ]"));
+#else
   ui_->playlist_summary->setMinimumWidth(QFontMetrics(font()).width("WW selected of WW tracks - [ WW:WW ]"));
+#endif
   ui_->status_bar_stack->setCurrentWidget(ui_->playlist_summary_page);
   connect(ui_->multi_loading_indicator, SIGNAL(TaskCountChange(int)), SLOT(TaskCountChanged(int)));
 
@@ -687,13 +741,15 @@ MainWindow::MainWindow(Application *app, SystemTrayIcon *tray_icon, OSD *osd, co
   connect(app_->player(), SIGNAL(Stopped()), ui_->widget_playing, SLOT(Stopped()));
   connect(app_->player(), SIGNAL(Error()), ui_->widget_playing, SLOT(Error()));
   connect(ui_->widget_playing, SIGNAL(ShowAboveStatusBarChanged(bool)), SLOT(PlayingWidgetPositionChanged(bool)));
+  connect(this, SIGNAL(AlbumCoverReady(Song, QUrl, QImage)), ui_->widget_playing, SLOT(AlbumCoverLoaded(Song, QUrl, QImage)));
+  connect(this, SIGNAL(SearchCoverInProgress()), ui_->widget_playing, SLOT(SearchCoverInProgress()));
 
   connect(ui_->action_console, SIGNAL(triggered()), SLOT(ShowConsole()));
   PlayingWidgetPositionChanged(ui_->widget_playing->show_above_status_bar());
 
   // Load theme
   // This is tricky: we need to save the default/system palette now,
-  // before loading user preferred theme (which will overide it), to be able to restore it later
+  // before loading user preferred theme (which will override it), to be able to restore it later
   const_cast<QPalette&>(Appearance::kDefaultPalette) = QApplication::palette();
   app_->appearance()->LoadUserTheme();
   StyleSheetLoader *css_loader = new StyleSheetLoader(this);
@@ -717,6 +773,7 @@ MainWindow::MainWindow(Application *app, SystemTrayIcon *tray_icon, OSD *osd, co
   connect(app_->playlist_manager()->sequence(), SIGNAL(ShuffleModeChanged(PlaylistSequence::ShuffleMode)), osd_, SLOT(ShuffleModeChanged(PlaylistSequence::ShuffleMode)));
 
   ScrobbleButtonVisibilityChanged(app_->scrobbler()->ScrobbleButton());
+  LoveButtonVisibilityChanged(app_->scrobbler()->LoveButton());
   ScrobblingEnabledChanged(app_->scrobbler()->IsEnabled());
 
   // Load settings
@@ -798,7 +855,7 @@ MainWindow::MainWindow(Application *app, SystemTrayIcon *tray_icon, OSD *osd, co
 
   RefreshStyleSheet();
 
-  qLog(Debug) << "Started";
+  qLog(Debug) << "Started" << QThread::currentThread();
   initialised_ = true;
 
   app_->scrobbler()->ConnectError();
@@ -855,11 +912,33 @@ void MainWindow::ReloadSettings() {
   bool enable_tidal = settings.value("enabled", false).toBool();
   settings.endGroup();
   if (enable_tidal)
-    ui_->tabs->EnableTab(tidal_search_view_);
+    ui_->tabs->EnableTab(tidal_view_);
   else
-    ui_->tabs->DisableTab(tidal_search_view_);
+    ui_->tabs->DisableTab(tidal_view_);
 #endif
 
+#ifdef HAVE_QOBUZ
+  settings.beginGroup(QobuzSettingsPage::kSettingsGroup);
+  bool enable_qobuz = settings.value("enabled", false).toBool();
+  settings.endGroup();
+  if (enable_qobuz)
+    ui_->tabs->EnableTab(qobuz_view_);
+  else
+    ui_->tabs->DisableTab(qobuz_view_);
+#endif
+
+#ifdef HAVE_SUBSONIC
+  settings.beginGroup(SubsonicSettingsPage::kSettingsGroup);
+  bool enable_subsonic = settings.value("enabled", false).toBool();
+  settings.endGroup();
+  if (enable_subsonic)
+    ui_->tabs->EnableTab(subsonic_view_);
+  else
+    ui_->tabs->DisableTab(subsonic_view_);
+#endif
+
+  ui_->tabs->ReloadSettings();
+
 }
 
 void MainWindow::ReloadAllSettings() {
@@ -873,10 +952,17 @@ void MainWindow::ReloadAllSettings() {
   osd_->ReloadSettings();
   collection_view_->ReloadSettings();
   ui_->playlist->view()->ReloadSettings();
+  app_->album_cover_loader()->ReloadSettings();
   album_cover_choice_controller_->ReloadSettings();
   if (cover_manager_.get()) cover_manager_->ReloadSettings();
 #ifdef HAVE_TIDAL
-  tidal_search_view_->ReloadSettings();
+  tidal_view_->ReloadSettings();
+#endif
+#ifdef HAVE_QOBUZ
+  qobuz_view_->ReloadSettings();
+#endif
+#ifdef HAVE_SUBSONIC
+  subsonic_view_->ReloadSettings();
 #endif
 
 }
@@ -887,6 +973,60 @@ void MainWindow::RefreshStyleSheet() {
   setStyleSheet(contents);
 }
 
+void MainWindow::SaveSettings() {
+
+  SaveGeometry();
+  SavePlaybackStatus();
+  ui_->tabs->SaveSettings(kSettingsGroup);
+  ui_->playlist->view()->SaveGeometry();
+  ui_->playlist->view()->SaveSettings();
+  app_->scrobbler()->WriteCache();
+
+  QSettings s;
+  s.beginGroup(kSettingsGroup);
+  s.setValue("search_for_cover_auto", album_cover_choice_controller_->search_cover_auto_action()->isChecked());
+  s.endGroup();
+
+}
+
+void MainWindow::Exit() {
+
+  ++exit_count_;
+
+  SaveSettings();
+
+  if (exit_count_ > 1) {
+    qApp->quit();
+  }
+  else {
+    if (app_->player()->engine()->is_fadeout_enabled()) {
+      // To shut down the application when fadeout will be finished
+      connect(app_->player()->engine(), SIGNAL(FadeoutFinishedSignal()), this, SLOT(DoExit()));
+      if (app_->player()->GetState() == Engine::Playing) {
+        app_->player()->Stop();
+        hide();
+        if (tray_icon_) tray_icon_->SetVisible(false);
+        return; // Don't quit the application now: wait for the fadeout finished signal
+      }
+    }
+    DoExit();
+  }
+
+}
+
+void MainWindow::DoExit() {
+
+  connect(app_, SIGNAL(ExitFinished()), this, SLOT(ExitFinished()));
+  app_->Exit();
+
+}
+
+void MainWindow::ExitFinished() {
+
+  qApp->quit();
+
+}
+
 void MainWindow::EngineChanged(Engine::EngineType enginetype) {
 
   ui_->action_equalizer->setEnabled(enginetype == Engine::EngineType::GStreamer || enginetype == Engine::EngineType::Xine);
@@ -905,6 +1045,10 @@ void MainWindow::MediaStopped() {
 
   ui_->action_play_pause->setEnabled(true);
 
+  ui_->action_love->setEnabled(false);
+  ui_->button_love->setEnabled(false);
+  if (tray_icon_) tray_icon_->LoveStateChanged(false);
+
   track_position_timer_->stop();
   track_slider_timer_->stop();
   ui_->track_slider->SetStopped();
@@ -917,6 +1061,8 @@ void MainWindow::MediaStopped() {
   song_ = Song();
   image_original_ = QImage();
 
+  app_->scrobbler()->ClearPlaying();
+
 }
 
 void MainWindow::MediaPaused() {
@@ -931,7 +1077,9 @@ void MainWindow::MediaPaused() {
   track_position_timer_->stop();
   track_slider_timer_->stop();
 
-  if (tray_icon_) tray_icon_->SetPaused();
+  if (tray_icon_) {
+    tray_icon_->SetPaused();
+  }
 
 }
 
@@ -963,6 +1111,9 @@ void MainWindow::MediaPlaying() {
   if (app_->scrobbler()->IsEnabled() && playlist && !playlist->nowplaying() && item->Metadata().is_metadata_good() && item->Metadata().length_nanosec() > 0) {
     app_->scrobbler()->UpdateNowPlaying(item->Metadata());
     playlist->set_nowplaying(true);
+    ui_->action_love->setEnabled(true);
+    ui_->button_love->setEnabled(true);
+    if (tray_icon_) tray_icon_->LoveStateChanged(true);
   }
 
 }
@@ -1179,13 +1330,13 @@ void MainWindow::closeEvent(QCloseEvent *event) {
   settings.endGroup();
 
   if (keep_running && event->spontaneous() && QSystemTrayIcon::isSystemTrayAvailable()) {
-    event->ignore();
     SetHiddenInTray(true);
   }
   else {
     Exit();
-    QApplication::quit();
   }
+  event->ignore();
+
 }
 
 void MainWindow::SetHiddenInTray(bool hidden) {
@@ -1194,7 +1345,7 @@ void MainWindow::SetHiddenInTray(bool hidden) {
 
   // Some window managers don't remember maximized state between calls to hide() and show(), so we have to remember it ourself.
   if (hidden) {
-    //was_maximized_ = isMaximized();
+    was_maximized_ = isMaximized();
     hide();
   }
   else {
@@ -1358,6 +1509,14 @@ void MainWindow::AddToPlaylist(QAction *action) {
 
 }
 
+void MainWindow::PlaylistMenuHidden() {
+
+  playlist_queue_->setVisible(true);
+  playlist_queue_play_next_->setVisible(true);
+  playlist_skip_->setVisible(true);
+
+}
+
 void MainWindow::PlaylistRightClick(const QPoint &global_pos, const QModelIndex &index) {
 
   QModelIndex source_index = app_->playlist_manager()->current()->proxy()->mapToSource(index);
@@ -1429,6 +1588,10 @@ void MainWindow::PlaylistRightClick(const QPoint &global_pos, const QModelIndex
   ui_->action_auto_complete_tags->setEnabled(false);
   ui_->action_auto_complete_tags->setVisible(false);
 #endif
+
+  ui_->action_rescan_songs->setEnabled(editable);
+  ui_->action_rescan_songs->setVisible(editable);
+
   // the rest of the read / write actions work only when there are no CUEs involved
   if (cue_selected) editable = 0;
 
@@ -1479,7 +1642,7 @@ void MainWindow::PlaylistRightClick(const QPoint &global_pos, const QModelIndex
   if (not_in_queue == 0) playlist_queue_->setIcon(IconLoader::Load("go-previous"));
   else playlist_queue_->setIcon(IconLoader::Load("go-next"));
 
-  if (in_skipped < selected) playlist_skip_->setIcon(IconLoader::Load("media-forward"));
+  if (in_skipped < selected) playlist_skip_->setIcon(IconLoader::Load("media-skip-forward"));
   else playlist_skip_->setIcon(IconLoader::Load("media-play"));
 
 
@@ -1576,6 +1739,25 @@ void MainWindow::PlaylistStopAfter() {
   app_->playlist_manager()->current()->StopAfter(playlist_menu_index_.row());
 }
 
+void MainWindow::RescanSongs() {
+
+  SongList songs;
+  PlaylistItemList items;
+
+  for (const QModelIndex& index : ui_->playlist->view()->selectionModel()->selection().indexes()) {
+    if (index.column() != 0) continue;
+    int row = app_->playlist_manager()->current()->proxy()->mapToSource(index).row();
+    PlaylistItemPtr item(app_->playlist_manager()->current()->item_at(row));
+    Song song = item->Metadata();
+
+    songs << song;
+    items << item;
+  }
+
+  app_->collection()->Rescan(songs);
+
+}
+
 void MainWindow::EditTracks() {
 
   SongList songs;
@@ -1646,10 +1828,12 @@ void MainWindow::RenumberTracks() {
 }
 
 void MainWindow::SongSaveComplete(TagReaderReply *reply, const QPersistentModelIndex &index) {
+
   if (reply->is_successful() && index.isValid()) {
     app_->playlist_manager()->current()->ReloadItems(QList<int>()<< index.row());
   }
   reply->deleteLater();
+
 }
 
 void MainWindow::SelectionSetValue() {
@@ -1794,6 +1978,7 @@ void MainWindow::CommandlineOptionsReceived(const quint32 instanceId, const QByt
 }
 
 void MainWindow::CommandlineOptionsReceived(const CommandlineOptions &options) {
+
   switch (options.player_action()) {
     case CommandlineOptions::Player_Play:
       if (options.urls().empty()) {
@@ -1827,6 +2012,15 @@ void MainWindow::CommandlineOptionsReceived(const CommandlineOptions &options) {
   }
 
   if (!options.urls().empty()) {
+
+#ifdef HAVE_TIDAL
+    for (const QUrl url : options.urls()) {
+      if (url.scheme() == "tidal" && url.host() == "login") {
+        emit AuthorisationUrlReceived(url);
+        return;
+      }
+    }
+#endif
     MimeData *data = new MimeData;
     data->setUrls(options.urls());
     // Behaviour depends on command line options, so set it here
@@ -1875,10 +2069,14 @@ void MainWindow::CommandlineOptionsReceived(const CommandlineOptions &options) {
 }
 
 void MainWindow::ForceShowOSD(const Song &song, const bool toggle) {
+
+  Q_UNUSED(song);
+
   if (toggle) {
     osd_->SetPrettyOSDToggleMode(toggle);
   }
   osd_->ReshowCurrentSong();
+
 }
 
 void MainWindow::Activate() {
@@ -1973,6 +2171,8 @@ void MainWindow::CopyFilesToDevice(const QList<QUrl> &urls) {
   else {
     QMessageBox::warning(this, tr("Error"), tr("None of the selected songs were suitable for copying to a device"));
   }
+#else
+  Q_UNUSED(urls);
 #endif
 }
 
@@ -2026,6 +2226,7 @@ void MainWindow::PlaylistOpenInBrowser() {
   }
 
   Utilities::OpenInFileBrowser(urls);
+
 }
 
 void MainWindow::PlaylistQueue() {
@@ -2190,11 +2391,13 @@ void MainWindow::PlaylistViewSelectionModelChanged() {
 }
 
 void MainWindow::PlaylistCurrentChanged(const QModelIndex &proxy_current) {
+
   const QModelIndex source_current =app_->playlist_manager()->current()->proxy()->mapToSource(proxy_current);
 
   // If the user moves the current index using the keyboard and then presses
   // F2, we don't want that editing the last column that was right clicked on.
   if (source_current != playlist_menu_index_) playlist_menu_index_ = QModelIndex();
+
 }
 
 void MainWindow::Raise() {
@@ -2202,34 +2405,19 @@ void MainWindow::Raise() {
   activateWindow();
 }
 
-#ifdef Q_OS_WIN32
-bool MainWindow::winEvent(MSG *msg, long*) {
+bool MainWindow::nativeEvent(const QByteArray &eventType, void *message, long *result) {
+
+  Q_UNUSED(eventType);
+  Q_UNUSED(result);
+
+#ifdef Q_OS_WIN
+  MSG *msg = static_cast<MSG*>(message);
   thumbbar_->HandleWinEvent(msg);
+#else
+  Q_UNUSED(message);
+#endif
+
   return false;
-}
-#endif  // Q_OS_WIN32
-
-void MainWindow::Exit() {
-
-  SaveGeometry();
-  SavePlaybackStatus();
-  ui_->tabs->SaveSettings(kSettingsGroup);
-  ui_->playlist->view()->SaveGeometry();
-  ui_->playlist->view()->SaveSettings();
-  app_->scrobbler()->WriteCache();
-
-  if (app_->player()->engine()->is_fadeout_enabled()) {
-    // To shut down the application when fadeout will be finished
-    connect(app_->player()->engine(), SIGNAL(FadeoutFinishedSignal()), qApp, SLOT(quit()));
-    if (app_->player()->GetState() == Engine::Playing) {
-      app_->player()->Stop();
-      hide();
-      if (tray_icon_) tray_icon_->SetVisible(false);
-      return; // Don't quit the application now: wait for the fadeout finished signal
-    }
-  }
-
-  qApp->quit();
 
 }
 
@@ -2243,7 +2431,7 @@ void MainWindow::AutoCompleteTags() {
     track_selection_dialog_->set_save_on_close(true);
 
     connect(tag_fetcher_.get(), SIGNAL(ResultAvailable(Song, SongList)), track_selection_dialog_.get(), SLOT(FetchTagFinished(Song, SongList)), Qt::QueuedConnection);
-    connect(tag_fetcher_.get(), SIGNAL(Progress(Song, QString)), track_selection_dialog_.get(), SLOT(FetchTagProgress(Song,QString)));
+    connect(tag_fetcher_.get(), SIGNAL(Progress(Song, QString)), track_selection_dialog_.get(), SLOT(FetchTagProgress(Song, QString)));
     connect(track_selection_dialog_.get(), SIGNAL(accepted()), SLOT(AutoCompleteTagsAccepted()));
     connect(track_selection_dialog_.get(), SIGNAL(finished(int)), tag_fetcher_.get(), SLOT(Cancel()));
     connect(track_selection_dialog_.get(), SIGNAL(Error(QString)), SLOT(ShowErrorDialog(QString)));
@@ -2353,20 +2541,19 @@ void MainWindow::ShowCover() {
 
 void MainWindow::SearchCoverAutomatically() {
 
-  QSettings s;
-  s.beginGroup(kSettingsGroup);
-  s.setValue("search_for_cover_auto", album_cover_choice_controller_->search_cover_auto_action()->isChecked());
-  s.endGroup();
   GetCoverAutomatically();
 
 }
 
-void MainWindow::AlbumArtLoaded(const Song &song, const QString&, const QImage &image) {
+void MainWindow::AlbumCoverLoaded(const Song &song, const QUrl &cover_url, const QImage &image) {
 
   if (song.effective_albumartist() != song_playing_.effective_albumartist() || song.effective_album() != song_playing_.effective_album() || song.title() != song_playing_.title()) return;
 
   song_ = song;
   image_original_ = image;
+
+  emit AlbumCoverReady(song, cover_url, image);
+
   GetCoverAutomatically();
 
 }
@@ -2375,14 +2562,18 @@ void MainWindow::GetCoverAutomatically() {
 
   // Search for cover automatically?
   bool search =
+               (song_.source() == Song::Source_LocalFile || song_.source() == Song::Source_Collection || song_.source() == Song::Source_CDDA) &&
                album_cover_choice_controller_->search_cover_auto_action()->isChecked() &&
                !song_.has_manually_unset_cover() &&
-               song_.art_automatic().isEmpty() &&
-               song_.art_manual().isEmpty() &&
+               !song_.art_automatic_is_valid() &&
+               !song_.art_manual_is_valid() &&
                !song_.effective_albumartist().isEmpty() &&
                !song_.effective_album().isEmpty();
 
-  if (search) album_cover_choice_controller_->SearchCoverAutomatically(song_);
+  if (search) {
+    album_cover_choice_controller_->SearchCoverAutomatically(song_);
+    emit SearchCoverInProgress();
+  }
 
 }
 
@@ -2391,12 +2582,24 @@ void MainWindow::ScrobblingEnabledChanged(bool value) {
 }
 
 void MainWindow::ScrobbleButtonVisibilityChanged(bool value) {
+
   ui_->button_scrobble->setVisible(value);
   ui_->action_toggle_scrobbling->setVisible(value);
   if (value) SetToggleScrobblingIcon(app_->scrobbler()->IsEnabled());
 
 }
 
+void MainWindow::LoveButtonVisibilityChanged(bool value) {
+
+  if (value)
+    ui_->widget_love->show();
+  else
+    ui_->widget_love->hide();
+
+  if (tray_icon_) tray_icon_->LoveVisibilityChanged(value);
+
+}
+
 void MainWindow::SetToggleScrobblingIcon(bool value) {
 
   if (value) {
@@ -2410,3 +2613,12 @@ void MainWindow::SetToggleScrobblingIcon(bool value) {
   }
 
 }
+
+void MainWindow::Love() {
+
+  app_->scrobbler()->Love();
+  ui_->button_love->setEnabled(false);
+  ui_->action_love->setEnabled(false);
+  if (tray_icon_) tray_icon_->LoveStateChanged(false);
+
+}
